/* SPDX-License-Identifier: GPL-2.0+ */

#ifndef _uapi_avm_linux_event_h_
#define _uapi_avm_linux_event_h_

/* nsc@avm: userspace headers net/if.h and linux/if.h bite each other.
 * To allow continuing fast-forward development we assume IFNAMSIZ to
 * stay 16; further we add a warning if neither <net/if.h> nor
 * <linux/if.h> is included.
 *
 * Iff one of these _is_ included before avm_event.h, we check whether
 * our IFNAMSIZ-stays-16 assumption is still valid.
 */

#define _avm_event_ifnamsiz 16

#ifndef IFNAMSIZ
# warning linux/avm_event.h requires pre-inclusion of <net/if.h> _or_ <linux/if.h>!
# define IFNAMSIZ _avm_event_ifnamsiz
# define _avm_event_supplied_ifnamsiz_ 1
#elif _avm_event_ifnamsiz != IFNAMSIZ
# error IFNAMSIZ constrained failed: IFNAMSIZ changed its value (was: 16 Byte)!
#endif

#include <avm/event/gen-types.h>


#define MAX_EVENT_CLIENT_NAME_LEN 32
#define MAX_EVENT_SOURCE_NAME_LEN 32
#define MAX_EVENT_NODE_NAME_LEN 32
#define MAX_AVM_EVENT_SOURCES 32
#define MAX_AVM_EVENT_NODES 32

/* This define identifies that the power manager can deliver cpu idle and temperature information
 * See the structures _avm_event_cpu_idle and _avm_event_temperature below.
 */
#define AVM_EVENT_HAVE_CPU_TEMPERATURE 1
/**
 * Aufteilung der Event typen (vorlaeufig in 8er Gruppen):
 *
 *  0 -  7:   WLAN
 *  8 - 15:   DSL
 * 16 - 23:
 * 24 - 31:
 * 32 - 39:   Ethernet
 * 40 - 47:
 * 48 - 55:
 * 56 - 63:
 */
#define AVM_EVENT_TYPE_WLAN_CLIENT_CHANGE 0
#define AVM_EVENT_TYPE_WLAN_ERROR         1


#if defined(CONFIG_AVM_EVENTNODE_PUMA6) || defined(CONFIG_AVM_EVENTNODE_PUMA7)

#if defined(CONFIG_AVM_EVENTNODE_PUMA6)
#define ARM_MEM_OFFSET 0x40000000
#else
#define ARM_MEM_OFFSET 0x0
#endif
int avm_event_node_established(
	void (*event_established_cb)(void *context, unsigned int param1,
				     unsigned int param2),
	void *context, unsigned int param1, unsigned int param2);
#define FLAG_REMOTELOG_REBOOT 0x1
#define FLAG_REMOTELOG_APPEND 0x2
#define FLAG_REMOTELOG_APPEND_FINISHED (0x4 | FLAG_REMOTELOG_APPEND)
void avm_event_send_log(enum _avm_logtype type, unsigned int flag,
			const char *buffer, unsigned int len);

/**
 * initial auszufuehren um net_notifier zu installieren
 */
void remotedev_net_init(char *remote_device);
/**
 * erst wenn alle server beendet kann netdev unregistriert werden !
 */
void remotedev_net_exit(void);

/**
 * Remote-Device-Server mit ETH-Protocol h_proto erzeugen
 * wenn auf der Remote-Seite ein open gemacht wird -> callback remotedev_open() wird getriggert
 * h_proto: ETH-Protokoll
 * ret: NULL Verbindung ablehnen, sonst rhandle
 * nachfolgend kommen alle Daten des remotedev per remotedev_rcv() herein
 * wenn Remote-Seite Device schliesst so erfolgt remotedev_close()
 *
 * Der Kontext der Callbacks ist fein saeuberlich serialisert ein rx-worker-thread
 */
int remotedev_server_create(
	unsigned short h_proto,
	/**
			 * Callback Remotseite macht open()
			 * conn:    Connection-Handle
			 * h_proto: ETH-Protokoll
			 * ret: NULL Verbindung ablehnen, sonst rhandle
			 */
	void *(*remotedev_open)(unsigned short h_proto),
	/**
			 * Callback RemoteSeite macht write()
			 * rhandle: Handle von remotedev_open()
			 * buf, len: die Daten
			 */
	void (*remotedev_rcv)(void *rhandle, unsigned char *buf,
			      unsigned int len),
	/**
			 * optional: Callback RemoteSeite macht read mit angeforderter Laenge
			 * rhandle: Handle von remotedev_open()
			 * len: die angeforderte Laenge
			 */
	void (*remotedev_demand)(void *rhandle, unsigned int len),
	/**
			 * Callback RemoteSeite: macht close()
			 * internal_error: 0 von Gegenseite, sonst internes Problem
			 */
	void (*remotedev_close)(void *rhandle, int internal_error));

/**
 * senden von Daten - koennen auf Remote-Seite dann per read aus Device geholt werden
 * rhandle: Handle vom open
 * ret: < 0 Fehler bzw. gesendete Byte (immer komplett)
 *
 * Kontext: nur Kernelthread/Workqueue erlaubt
 */
int remotedev_write(void *rhandle, unsigned short h_proto, unsigned char *buf,
		    unsigned int len);

/**
 * Remote-Device-Server mit ETH-Protocol h_proto abraeumen
 */
void remotedev_server_delete(unsigned short h_proto);

/**
 * ist in den arm2atom/atom2arm-irq einzuklinken:
 */
unsigned int handle_remotecpuirq(unsigned int cause);

/**
 * Schnittstelle um Remote-Trigger-Irqs (bis zu 16) generisch nutzen zu koennen
 * allerdings ist IRQ 0 fuer REBOOT-Signalisierung reserviert
 * Returncode vom Handler wird NICHT ausgewertet (dient nur zur Komaptibilitt mit request_irq())
 * -> gleicher Handler verwendbar
 */
#define AVM_EVENT_REMOTECPU_IRQ   0x1
#define AVM_CPUCONN_REMOTECPU_IRQ 0x2
#define AVM_PCMLINK_REMOTECPU_IRQ 0x3
/**
 * like request_irq ...
 */
int request_remotecpu_irqhandler(int irq,
				 int (*remotecpu_handler)(int irq, void *ref),
				 void *ref);
int free_remotecpu_irqhandler(int irq, void *ref);
void enable_remotecpu_irqhandler(int irq);
void disable_remotecpu_irqhandler(int irq);

/**
 * Remote-CPU triggern
 */
void trigger_remotecpuirq(unsigned int irq);
#else /*--- #if defined(CONFIG_AVM_EVENTNODE_PUMA6) ---*/
/**
 */
static __inline__ int avm_event_node_established(
	void (*event_established_cb)(void *context, unsigned int param1,
				     unsigned int param2),
	void *context, unsigned int param1, unsigned int param2)
{
	if (event_established_cb)
		event_established_cb(context, param1, param2);
	return 0;
}
#endif
/**
 * user event structures
 */
#endif /*--- #ifndef _uapi_avm_linux_event_h_ ---*/
